#include <libmemcached/memcached.h>
#include <libmemcached/memcached_server.h>
#include <ch.h>

static ChInterp_t interp;

////////////////////////
//memcached_dump_chdl
////////////////////////


static void* memcached_dump_chdl_funptr;


static void* memcached_dump_chdl_funarg(memcached_st *ptr, const char *key, size_t key_length, void *context)
{
  void* retval;
  Ch_CallFuncByAddr(interp, memcached_dump_chdl_funptr, &retval, ptr, key, key_length, context);
  return retval;
}


EXPORTCH memcached_return memcached_dump_chdl(void *varg) {
    //ChInterp_t interp;
    ChVaList_t ap;
    memcached_st *ptr;
    memcached_dump_func *function_ch, *function_c = NULL;
    void *context;
    uint32_t number_of_callbacks;
    memcached_return retval;

    Ch_VaStart(interp, ap, varg);
    ptr = Ch_VaArg(interp, ap, memcached_st *);
    function_ch = Ch_VaArg(interp, ap, memcached_dump_func *);
    
    memcached_dump_chdl_funptr = (void*)function_ch;
    if (function_ch!=NULL)
      function_c = (memcached_dump_func*)memcached_dump_chdl_funarg;
    
    context = Ch_VaArg(interp, ap, void *);
    number_of_callbacks = Ch_VaArg(interp, ap, uint32_t);
    retval = memcached_dump(ptr, function_c, context, number_of_callbacks);
    Ch_VaEnd(interp, ap);
    return retval;
}




///////////////////////////
//memcached_set_memory_allocators_chdl
///////////////////////////

static void* memcached_malloc_function_chdl_funptr;
static void* memcached_free_function_chdl_funptr;
static void* memcached_realloc_function_chdl_funptr;
static void* memcached_calloc_function_chdl_funptr;

static void* memcached_malloc_function_chdl_funarg(memcached_st *ptr, const size_t size)
{
  void* retval;
  Ch_CallFuncByAddr(interp, memcached_malloc_function_chdl_funptr, &retval, ptr, size);
  return retval;
}

static void memcached_free_function_chdl_funarg(memcached_st *ptr, void *mem)
{
  Ch_CallFuncByAddr(interp, memcached_free_function_chdl_funptr, ptr, mem);
}

static void* memcached_realloc_function_chdl_funarg(memcached_st *ptr, void *mem, const size_t size)
{
  void* retval;
  Ch_CallFuncByAddr(interp, memcached_realloc_function_chdl_funptr, &retval, ptr, mem, size);
  return retval;
}

static void* memcached_calloc_function_chdl_funarg(memcached_st *ptr, size_t nelem, const size_t elsize)
{
  void* retval;
  Ch_CallFuncByAddr(interp, memcached_calloc_function_chdl_funptr, &retval, ptr, nelem, elsize);
  return retval;
}


EXPORTCH memcached_return memcached_set_memory_allocators_chdl(void *varg) {
    //ChInterp_t interp;
    ChVaList_t ap;
    memcached_st *ptr;
    memcached_malloc_function mem_malloc_CH, mem_malloc_C=NULL;
    memcached_free_function mem_free_CH, mem_free_C=NULL;
    memcached_realloc_function mem_realloc_CH, mem_realloc_C=NULL;
    memcached_calloc_function mem_calloc_CH, mem_calloc_C=NULL;
    memcached_return retval;

    Ch_VaStart(interp, ap, varg);
    ptr = Ch_VaArg(interp, ap, memcached_st *);
    
    mem_malloc_CH = Ch_VaArg(interp, ap, memcached_malloc_function);
    memcached_malloc_function_chdl_funptr = (void*)mem_malloc_CH;
    if (mem_malloc_CH!=NULL)
      mem_malloc_C = (memcached_malloc_function)memcached_malloc_function_chdl_funarg;
    
    mem_free_CH = Ch_VaArg(interp, ap, memcached_free_function);
    memcached_free_function_chdl_funptr = (void*)mem_free_CH;
    if (mem_free_CH!=NULL)
      mem_free_C = (memcached_free_function)memcached_free_function_chdl_funarg;
    
    mem_realloc_CH = Ch_VaArg(interp, ap, memcached_realloc_function);
    memcached_realloc_function_chdl_funptr = (void*)mem_realloc_CH;
    if (mem_realloc_CH!=NULL)
      mem_realloc_C = (memcached_realloc_function)memcached_realloc_function_chdl_funarg;
    
    mem_calloc_CH = Ch_VaArg(interp, ap, memcached_calloc_function);
    memcached_calloc_function_chdl_funptr = (void*)mem_calloc_CH;
    if (mem_calloc_CH!=NULL)
      mem_calloc_C = (memcached_calloc_function)memcached_calloc_function_chdl_funarg;
    
    retval = memcached_set_memory_allocators(ptr, mem_malloc_C, mem_free_C, mem_realloc_C, mem_calloc_C);
    Ch_VaEnd(interp, ap);
    return retval;
}


////////////////////////
//memcached_get_memory_allocators_chdl
////////////////////////

EXPORTCH void memcached_get_memory_allocators_chdl(void *varg) {
    //ChInterp_t interp;
    ChVaList_t ap;
    memcached_st *ptr;
    memcached_malloc_function *mem_malloc_CH, *mem_malloc_C=NULL;
    memcached_free_function *mem_free_CH, *mem_free_C=NULL;
    memcached_realloc_function *mem_realloc_CH, *mem_realloc_C=NULL;
    memcached_calloc_function *mem_calloc_CH, *mem_calloc_C=NULL;

    Ch_VaStart(interp, ap, varg);
    ptr = Ch_VaArg(interp, ap, memcached_st *);
    mem_malloc_CH = Ch_VaArg(interp, ap, memcached_malloc_function*);
    memcached_malloc_function_chdl_funptr = (void*)mem_malloc_CH;
    if (mem_malloc_CH!=NULL)
      mem_malloc_C = (memcached_malloc_function*)memcached_malloc_function_chdl_funarg;
    
    mem_free_CH = Ch_VaArg(interp, ap, memcached_free_function*);
    memcached_free_function_chdl_funptr = (void*)mem_free_CH;
    if (mem_free_CH!=NULL)
      mem_free_C = (memcached_free_function*)memcached_free_function_chdl_funarg;
    
    mem_realloc_CH = Ch_VaArg(interp, ap, memcached_realloc_function*);
    memcached_realloc_function_chdl_funptr = (void*)mem_realloc_CH;
    if (mem_realloc_CH!=NULL)
      mem_realloc_C = (memcached_realloc_function*)memcached_realloc_function_chdl_funarg;
    
    mem_calloc_CH = Ch_VaArg(interp, ap, memcached_calloc_function*);
    memcached_calloc_function_chdl_funptr = (void*)mem_calloc_CH;
    if (mem_calloc_CH!=NULL)
      mem_calloc_C = (memcached_calloc_function*)memcached_calloc_function_chdl_funarg;
    
    memcached_get_memory_allocators(ptr, mem_malloc_C, mem_free_C, mem_realloc_C, mem_calloc_C);
    Ch_VaEnd(interp, ap);
}


//////////////////
//memcached_fetch_execute_chdl
//////////////////

static void* memcached_execute_function_chdl_funptr;
static memcached_return memcached_execute_function_chdl_funarg(memcached_st *ptr, memcached_result_st *result, void *context)
{
  memcached_return retval;
  Ch_CallFuncByAddr(interp, memcached_execute_function_chdl_funptr, &retval, ptr, result, context);
  return retval;
}

EXPORTCH memcached_return memcached_fetch_execute_chdl(void *varg) {
    //ChInterp_t interp;
    ChVaList_t ap;
    memcached_st *ptr;
    memcached_execute_function *callback_CH, *callback_C=NULL;
    void *context;
    unsigned int number_of_callbacks;
    memcached_return retval;

    Ch_VaStart(interp, ap, varg);
    ptr = Ch_VaArg(interp, ap, memcached_st *);
    
    callback_CH = Ch_VaArg(interp, ap, memcached_execute_function *);
    memcached_execute_function_chdl_funptr = (void*)callback_CH;
    if (callback_CH!=NULL)
      callback_C = (memcached_execute_function*)memcached_execute_function_chdl_funarg;
    
    context = Ch_VaArg(interp, ap, void *);
    number_of_callbacks = Ch_VaArg(interp, ap, unsigned int);
    retval = memcached_fetch_execute(ptr, callback_C, context, number_of_callbacks);
    Ch_VaEnd(interp, ap);
    return retval;
}



///////////////////
//memcached_server_cursor_chdl
///////////////////

static void* memcached_server_function_chdl_funptr;
static memcached_return memcached_server_function_chdl_funarg(memcached_st *ptr, memcached_server_st *server, void *context)
{
  memcached_return retval;
  Ch_CallFuncByAddr(interp, memcached_server_function_chdl_funptr, &retval, ptr, server, context);
  return retval;
}

EXPORTCH memcached_return memcached_server_cursor_chdl(void *varg) {
    //ChInterp_t interp;
    ChVaList_t ap;
    memcached_st *ptr;
    memcached_server_function *callback_CH, *callback_C = NULL;
    void *context;
    unsigned int number_of_callbacks;
    memcached_return retval;

    Ch_VaStart(interp, ap, varg);
    ptr = Ch_VaArg(interp, ap, memcached_st *);
    
    callback_CH = Ch_VaArg(interp, ap, memcached_server_function *);
    memcached_server_function_chdl_funptr = (void*)callback_CH;
    if (callback_CH!=NULL)
      callback_C = (memcached_server_function*)memcached_server_function_chdl_funarg;

    
    context = Ch_VaArg(interp, ap, void *);
    number_of_callbacks = Ch_VaArg(interp, ap, unsigned int);
    retval = memcached_server_cursor(ptr, callback_C, context, number_of_callbacks);
    Ch_VaEnd(interp, ap);
    return retval;
}
