#!/bin/ch

// this script will generate the ch package

#include <unistd.h>   // for access()
#include <dlfcn.h>    // for dlopen()
#include <chshell.h>  // for chinfo()

int addChdlHeader(char *headerfile, char *macro, char *chdlheader);

string_t pkgname="chmemcached";          // define package name
string_t libname="memcached";              // assume the lib is libsql.so 

string_t lib_header_dir;             // library header files directory 
string_t createchf_dir="chfcreate/"; // library directory containing chf files
string_t c_dir ="c/";                // directory  to generate .dl file
string_t include_dir="include/libmemcached/";     // header files used for Ch 
string_t demo_dir="demos/";          // directory  to generate .dl file
string_t cwd = _cwd;                 // the current working directory

chinfo_t info;

//make sure pgkcreate.ch is run from the current working directory
if(access("pkgcreate.ch", R_OK)) {
    echo Run ./pkgcreate.ch in the current directory.
    exit(-1);
}

// make sure you are running ch version workable with this script.
chinfo(&info);
if ((info.vermajor*100+ info.verminor*10 +  info.vermicro) < 501) {
   echo "To run this script, you need to install Ch version 5.0.1.12201 or higher"
   echo "You can download the latest version from http://www.softintegration.com/download"
   exit(-1);
}

/*  
    we assume the library header files are either 
    in /usr/local/include or /usr/include 
*/
if (access("/usr/local/include/libmemcached/memcached.h", R_OK ) == 0)
{
	echo "we assume memcache header files are located in /usr/local/include";
	lib_header_dir="/usr/local/include/libmemcached";
}
else if  (access("/usr/include/libmemcached/memcached.h", R_OK ) == 0)
{
	echo "we assume memcache header files are located in /usr/include";
	lib_header_dir = "/usr/include/libmemcached";
}
else
{
	echo "Error: unable to find memcached header files.";
        echo "Please download memcached source code and install it first.";
	exit(-1);
}

/* clean up existing chunixodbc directory  */
if (access(pkgname, R_OK) ==0)
{
        echo "Clean up package directory ${pkgname} now";
        rm -rf $pkgname;
        rm -rf $include_dir
}

// create the directory for distribution
mkdir $pkgname $pkgname/dl $pkgname/lib $pkgname/include $pkgname/include/libmemcached $pkgname/demos
mkdir -p $include_dir

echo copy library header files for ch
//cp $lib_header_dir/memcached.h $include_dir 
cp $lib_header_dir/*.h $include_dir 
/**/
// needs special handling for sql.h file used for Ch
char *macro ="#define __MEMCACHED_H__";

string_t chdlheader =stradd(
  "\n\n",
  "#if defined(_CH_) \n"
  "#pragma package <", pkgname, "> \n",
  "#include <chdl.h> \n",
  "LOAD_CHDL(", libname, ")\n",
  "#endif \n");

// copy the library header file with the change
// it will add chdlheader under "#define _SQL_H"; in sql.h
addChdlHeader(stradd(include_dir, "memcached.h"), macro, chdlheader);

echo generate header files containing exported functions only
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "memcached.h"), stradd(createchf_dir, "memcached.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "memcached_get.h"), stradd(createchf_dir, "memcached_get.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "memcached_result.h"), stradd(createchf_dir, "memcached_result.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "memcached_server.h"), stradd(createchf_dir, "memcached_server.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "memcached_storage.h"), stradd(createchf_dir, "memcached_storage.h"), NULL);
processhfile("LIBMEMCACHED_API", 0,  ";",   stradd(include_dir, "memcached_string.h"), stradd(createchf_dir, "memcached_string.h"), NULL);

//these functions are handmade as arguments - pointer to functions
removeFuncProto("chfcreate/memcached.h", "memcached_set_memory_allocators", 0);
removeFuncProto("chfcreate/memcached.h", "memcached_get_memory_allocators", 0);
removeFuncProto("chfcreate/memcached.h", "memcached_fetch_execute", 0);
removeFuncProto("chfcreate/memcached_server.h", "memcached_server_cursor", 0);
removeFuncProto("chfcreate/memcached.h", "memcached_dump", 0);

echo generated header files
//exit(-1);


echo generating *.chf and *_chdl.c files
echo process memcache.h
c2chf $createchf_dir/memcached.h -i c libmemcached/memcached.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/memcached_get.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/memcached_get.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/memcached_result.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/memcached_result.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/memcached_server.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/memcached_server.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/memcached_storage.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/memcached_storage.h -o c ${c_dir} -o chf ${pkgname}/lib
c2chf $createchf_dir/memcached_string.h -h _Ch${libname}_handle -i c libmemcached/memcached.h -i c libmemcached/memcached_string.h -o c ${c_dir} -o chf ${pkgname}/lib


echo patching special .chf files ...
cp -f chfhandmade/*.chf $pkgname/lib
cp -f chfhandmade/*.c $c_dir

cd ${c_dir}
echo clean up ${c_dir} object files
make clean
make

echo test if .dl file is created successfully
string_t libmemcachedl = stradd(`pwd`, "/libmemcached.dl");

if ( strcmp(`uname`,"Linux") ==0 )
{
    chcon -t texrel_shlib_t $libmemcachedl  >/dev/null 2>&1
}

if ( dlopen(libmemcachedl, 1) == NULL)
{
    echo "error in libmemcached.dl";
    printf("%s \n", dlerror());
    exit(-1);
}


// go back to previous directory
cd $cwd

echo "libmemcache.dl is successfully created";

echo copy the created files
cp $c_dir/lib${libname}.dl $pkgname/dl/
cp $include_dir/*.h $pkgname/include/libmemcached
cp -p $demo_dir/* $pkgname/demos/

echo "${pkgname} is successfully created";

/*******************************************************************
 * * This function will add the required preprocessing directives
 * * needed by Ch to a header file.
 * * headerfile:   header file to be changed
 * * macro:        token to be recoganized for insertion of chdlheader
 * * chdlheader:   preprocessing directives to be inserted
 * *******************************************************************/
int addChdlHeader(char *headerfile, char *macro, char *chdlheader)
{
   char *stop_ptr;
   string_t tempfile, token;
   string_t result;

   tempfile = ``cat $headerfile``;
   token = strstr( tempfile, macro);
   stop_ptr = strstr(token, "\n");
   strncpy(result, tempfile, strlen(tempfile)-strlen(stop_ptr));
   result=stradd(result, chdlheader);
   result=stradd(result, stop_ptr);
   
   FILE *stream;
   if(!(stream = fopen("_tmpfile", "w")))
   {
      perror("_tmpfile");
      return -1;
   }
   fprintf(stream, "%s", result);
   fclose(stream);
   mv -f _tmpfile $(headerfile)

   return 0;

}
